package schema

import (
	"entgo.io/ent"
	"entgo.io/ent/dialect/entsql"
	"entgo.io/ent/schema"
	"entgo.io/ent/schema/field"
	"entgo.io/ent/schema/index"
	"github.com/suyuan32/simple-admin-common/orm/ent/mixins"
	"wechat-api/ent/schema/localmixin"
)

type MessageRecords struct {
	ent.Schema
}

func (MessageRecords) Fields() []ent.Field {
	return []ent.Field{
		field.String("bot_wxid").
			Annotations(entsql.WithComments(true)).
			Comment("机器人微信 id"),
		field.Int("contact_id").
			Annotations(entsql.WithComments(true)).
			Comment("联系人 id"),
		field.Int("contact_type").Default(1).
			Annotations(entsql.WithComments(true)).
			Comment("类型：1好友，2群组，3企业微信联系人"),
		field.String("contact_wxid").Default("").
			Annotations(entsql.WithComments(true)).
			Comment("接收方微信 id"),
		field.Int("content_type").Default(1).
			Annotations(entsql.WithComments(true)).
			Comment("内容类型 1 文本 2 文件"),
		field.String("content").Default("").
			Annotations(entsql.WithComments(true)).
			Comment("发送内容"),
		field.String("error_detail").Default("").
			Annotations(entsql.WithComments(true)).
			Comment("异常原因"),
		field.Time("send_time").Optional().
			Annotations(entsql.WithComments(true)).
			Comment("发送时间"),
		field.Int("source_type").Default(1).
			Annotations(entsql.WithComments(true)).
			Comment("源类型 1 点发 2 群发 3 SOP"),
		field.Int("source_id").Default(1).
			Annotations(entsql.WithComments(true)).
			Comment("源 ID"),
		field.Int("sub_source_id").Default(1).
			Annotations(entsql.WithComments(true)).
			Comment("次源 ID"),
	}
}

func (MessageRecords) Mixin() []ent.Mixin {
	return []ent.Mixin{
		mixins.IDMixin{},
		mixins.StatusMixin{},
		localmixin.SoftDeleteMixin{},
	}
}

func (MessageRecords) Indexes() []ent.Index {
	return []ent.Index{
		index.Fields("source_type"),
	}
}

func (MessageRecords) Edges() []ent.Edge {
	return nil
}

func (MessageRecords) Annotations() []schema.Annotation {
	return []schema.Annotation{entsql.Annotation{Table: "message_records"}}
}
